/*
 * Copyright (c) 1996-2009 Barton P. Miller
 * 
 * We provide the Paradyn Parallel Performance Tools (below
 * described as "Paradyn") on an AS IS basis, and do not warrant its
 * validity or performance.  We reserve the right to update, modify,
 * or discontinue this software at any time.  We shall have no
 * obligation to supply such updates or modifications or any other
 * form of support to you.
 * 
 * By your use of Paradyn, you understand and agree that we (or any
 * other person or entity with proprietary rights in Paradyn) are
 * under no obligation to provide either maintenance services,
 * update services, notices of latent defects, or correction of
 * defects for Paradyn.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "process.h"
#include "EventHandler.h"
#include "mailbox.h"
#include "signalgenerator.h"
#include "inst.h"
#include "instP.h"
#include "instPoint.h"
#include "function.h" // int_function
#include "codeRange.h"
#include "dyn_thread.h"
#include "miniTramp.h"

#include "mapped_module.h"

#include "BPatch_libInfo.h"
#include "BPatch_asyncEventHandler.h"
#include "BPatch.h"
#include "BPatch_thread.h"
#include "BPatch_function.h"
#include "BPatch_parRegion.h"
#if defined(cap_instruction_api)
#include "instructionAPI/h/Instruction.h"
#include "instructionAPI/h/InstructionDecoder.h"
#else
#include "InstrucIter.h"
#endif
#include "callbacks.h"

BPatch_parRegion::BPatch_parRegion(int_parRegion * _parReg, BPatch_function * _func)
{
  parReg = _parReg;
  func = _func;
  instructions = NULL;
}

BPatch_parRegion::~BPatch_parRegion()
{  }

void BPatch_parRegion::printDetails()
{
  parReg->printDetails();
}
BPatch_Vector<BPatch_instruction*> *BPatch_parRegion::getInstructionsInt(void) {

#if defined(cap_instruction_api)
	return NULL;
#else
  if (!instructions) {

    instructions = new BPatch_Vector<BPatch_instruction*>;

    InstrucIter ii(this);
    
    while(ii.hasMore()) {

      BPatch_instruction *instr = ii.getBPInstruction();
      instructions->push_back(instr);
      ii++;
    }
    
  }

  return instructions;
#endif
}

#if defined(cap_instruction_api)
bool BPatch_parRegion::getInstructionsInt(std::vector<InstructionAPI::Instruction::Ptr>& insns) {
  using namespace InstructionAPI;
  const unsigned char* buffer = 
  (const unsigned char*)(lowlevel_region()->intFunc()->proc()->getPtrToInstruction(getStartAddress()));
  
  InstructionDecoder d(buffer, size());
  d.setMode(lowlevel_region()->intFunc()->proc()->getAddressWidth() == 8);
  Instruction::Ptr curInsn = d.decode();
  while(curInsn && curInsn->isValid())
  {
    insns.push_back(curInsn);
    curInsn = d.decode();
  }
  return !insns.empty();
  
}
#else
bool BPatch_parRegion::getInstructionsInt(std::vector<InstructionAPI::Instruction::Ptr>&) {
  return false;
}
#endif // defined(cap_instruction_api)

unsigned long BPatch_parRegion::getStartAddressInt() CONST_EXPORT 
{
   return parReg->firstInsnAddr();
}

unsigned long BPatch_parRegion::getEndAddressInt() CONST_EXPORT
{
   return parReg->endAddr();
}

unsigned BPatch_parRegion::sizeInt() CONST_EXPORT
{
   return getEndAddress() - getStartAddress();
}

int BPatch_parRegion::getClauseInt(const char * key) 
{  
  return  parReg->getClause(key); 

}

int BPatch_parRegion::replaceOMPParameterInt(const char * key, int value)
{
  return parReg->replaceOMPParameter(key,value);
}

